@@ -3,10 +3,10 @@ Functionality to load a CLDF dataset into a sqlite db.
 
 To make the resulting SQLite database useful without access to the datasets metadata, we
 use terms of the CLDF ontology for database objects as much as possible, i.e.
-- table names are component names (e.g. "ValueTable" for a table with propertyUrl
+- table names are component names (e.g. "ValueTable" for a table with propertyUrl \
   http://cldf.clld.org/v1.0/terms.rdf#ValueTable)
-- column names are property names, prefixed with "cldf_" (e.g. a column with propertyUrl
-  http://cldf.clld.org/v1.0/terms.rdf#id will be "cldf_id" in the database)
+- column names are property names, prefixed with "cldf" + UNDERSCORE (e.g. a column with \
+  propertyUrl http://cldf.clld.org/v1.0/terms.rdf#id will be "cldf_id" in the database)
 
 This naming scheme also extends to automatically created association tables. I.e. when a
 table specifies a list-valued foreign key, an association table is created to implement this
@@ -17,6 +17,8 @@ many-to-many relationship. The name of the association table is the concatenatio
 E.g. a list-valued foreign key from the FormTable to the ParameterTable will result in an
 association table
 
+.. code-block:: sql
+
   CREATE TABLE `FormTable_ParameterTable` (
     `FormTable_cldf_id` TEXT,
     `ParameterTable_cldf_id` TEXT,
@@ -27,6 +29,8 @@ association table
 
 while a list-valued foreign key to a custom table may result in something like this
 
+.. code-block:: sql
+
   CREATE TABLE `FormTable_custom.csv` (
     `FormTable_cldf_id` TEXT,
     `custom.csv_id` TEXT,
@@ -35,6 +39,7 @@ while a list-valued foreign key to a custom table may result in something like t
     FOREIGN KEY(`custom.csv_id`) REFERENCES `custom.csv`(`id`) ON DELETE CASCADE
   );
 """
+import typing
 import pathlib
 import functools
 import collections
@@ -46,6 +51,8 @@ import csvw.db
 from pycldf.terms import TERMS
 from pycldf.sources import Reference, Sources, Source
 
+__all__ = ['Database']
+
 PRIMARY_KEY_NAMES = ['id', 'pk']
 BIBTEX_FIELDS = [
     'address',  # Publisher's address
@@ -79,11 +86,14 @@ BIBTEX_FIELDS = [
 
 @attr.s
 class TableTranslation(object):
+    """
+    Specifies column name translations for a table.
+    """
     name = attr.ib(default=None)
     columns = attr.ib(default=attr.Factory(dict))
 
 
-def translate(d, table, col=None):
+def translate(d: typing.Dict[str, TableTranslation], table: str, col=None) -> str:
     """
     Translate a db object name.
 
@@ -110,9 +120,18 @@ def clean_bibtex_key(s):
 
 
 class Database(csvw.db.Database):
+    """
+    Extend the functionality provided by `csvw.db.Database` by
+
+    - providing consistent naming of schema objects according to CLDF semantics,
+    - integrating sources into the DB schema.
+    """
     source_table_name = 'SourceTable'
 
     def __init__(self, dataset, **kw):
+        """
+        :param dataset: a `pycldf.Dataset` instance.
+        """
         self.dataset = dataset
         self._retranslate = collections.defaultdict(dict)
         self._source_cols = ['id', 'genre'] + BIBTEX_FIELDS
@@ -221,7 +240,10 @@ class Database(csvw.db.Database):
         return csvw.db.Database.write(
             self, _force=False, _exists_ok=False, _skip_extra=True, **items)
 
-    def write_from_tg(self, _force=False, _exists_ok=False):
+    def write_from_tg(self, _force: bool = False, _exists_ok: bool = False):
+        """
+        Write the data from `self.dataset` to the database.
+        """
         items = {
             tname: list(t.iterdicts())
             for tname, t in self.tg.tabledict.items() if tname != self.source_table_name}
@@ -233,7 +255,10 @@ class Database(csvw.db.Database):
             items[self.source_table_name].append(item)
         return self.write(_force=_force, _exists_ok=_exists_ok, **items)
 
-    def query(self, sql, params=None):
+    def query(self, sql: str, params=None) -> list:
+        """
+        Run `sql` on the database, returning the list of results.
+        """
         with self.connection() as conn:
             cu = conn.execute(sql, params or ())
             return list(cu.fetchall())
@@ -257,12 +282,12 @@ class Database(csvw.db.Database):
                 item[attr_] = round(item[attr_], precision)
         return item
 
-    def to_cldf(self, dest, mdname='cldf-metadata.json', coordinate_precision=4):
+    def to_cldf(self, dest, mdname='cldf-metadata.json', coordinate_precision=4) -> pathlib.Path:
         """
         Write the data from the db to a CLDF dataset according to the metadata in `self.dataset`.
 
-        :param dest:
-        :param mdname:
+        :param dest: Destination directory for the CLDF data.
+        :param mdname: Name to use for the CLDF metadata file.
         :return: path of the metadata file
         """
         dest = pathlib.Path(dest)
